@@ -1,3 +1,4 @@
+#! /usr/bin/env python
 from __future__ import print_function
 
 import calendar
@@ -29,7 +30,7 @@ def login_required(f):
 
 class Browser(object):
 
-    def __init__(self, username, password):
+    def __init__(self, username=None, password=None):
         self.session = requests.Session()
         self.username = username
         self.password = password
@@ -175,7 +176,7 @@ def check_for_sanity(row, month):
             pass
 
 
-def format_usage_info(info, format):
+def format_usage_info(info, format, show_remaining):
     if format == 'json':
         return json.dumps(info, indent=2)
     elif format == 'human':
@@ -184,45 +185,70 @@ def format_usage_info(info, format):
             output.append('  No data')
         for row in info['usage']:
             output.append(u'  {} {}'.format(row['type'].capitalize(),
-                                            format_quota_used(row)))
+                                            format_quota_used(row,
+                                                              show_remaining)))
         return '\n'.join(output)
     else:
         raise RuntimeError('Unsupported format: {!r}'.format(format))
 
 
-def format_quota_used(info):
+def format_quota_used(info, show_remaining):
     formatter = unit_formatters.get(info['type'], identity)
-    return '{} / {}'.format(formatter(info['used']), formatter(info['quota']))
+    val = info['used'] if not show_remaining else info['quota'] - info['used']
+    return '{} / {}'.format(formatter(val), formatter(info['quota']))
 
 
 @click.group()
 @click.option('--debug/--no-debug', default=False)
-@click.option('--username', help='olleh.com username')
-@click.option('--password', help='olleh.com password')
-@click.pass_context
-def cli(ctx, debug, username, password):
+def cli(debug):
     """Unofficial command line tool for olleh.com. To avoid typing username
     and password every time or to automate the script, you can set environment
-    variables OLLEH_USERNAME and OLLEH_PASSWORD. Actually every parameter to
-    this tool can be set from environment variables prefixed "OLLEH_".
+    variables OLLEH_USERNAME and OLLEH_PASSWORD.
 
     """
     if debug:
         logging.basicConfig(level=logging.DEBUG)
-    ctx.obj = Browser(username, password)
 
 
-def catch_login_error(f):
-    @functools.wraps(f)
-    @click.pass_context
-    def wrapper(ctx, *args, **kwargs):
-        try:
-            return ctx.invoke(f, *args, **kwargs)
-        except LoginError:
-            print('Login failed (username: {!r})'.format(ctx.obj.username),
-                  file=sys.stderr)
-            ctx.exit(1)
-    return wrapper
+pass_browser = click.make_pass_decorator(Browser, ensure=True)
+
+
+def browser_option(f):
+
+    def username_callback(ctx, param, value):
+        browser = ctx.ensure_object(Browser)
+        browser.username = value
+        return browser
+
+    def password_callback(ctx, param, value):
+        browser = ctx.ensure_object(Browser)
+        browser.password = value
+        return browser
+
+    def catch_login_error(f):
+        @functools.wraps(f)
+        @click.pass_context
+        def wrapper(ctx, *args, **kwargs):
+            try:
+                return ctx.invoke(f, *args, **kwargs)
+            except LoginError:
+                print('Login failed (username: {!r})'.format(ctx.obj.username),
+                      file=sys.stderr)
+                ctx.exit(1)
+        return wrapper
+
+    username_option = click.option('--username', metavar='USERNAME',
+                                   envvar='OLLEH_USERNAME',
+                                   prompt=True, expose_value=False,
+                                   callback=username_callback,
+                                   help='olleh.com username')
+    password_option = click.option('--password', metavar='PASSWORD',
+                                   envvar='OLLEH_PASSWORD',
+                                   prompt=True, hide_input=True,
+                                   expose_value=False,
+                                   callback=password_callback,
+                                   help='olleh.com password')
+    return username_option(password_option(catch_login_error(f)))
 
 
 @cli.command(short_help='print your mobile phone usage')
@@ -231,9 +257,11 @@ def catch_login_error(f):
 @click.option('--from-month', type=Month, metavar='YYYY-MM')
 @click.option('--format', type=click.Choice(['human', 'json']),
               default='human')
-@click.pass_obj
-@catch_login_error
-def usage(browser, month, from_month, format):
+@click.option('--remaining', is_flag=True, default=False,
+              help='Show remaining quota instead of used (for --format human)')
+@browser_option
+@pass_browser
+def usage(browser, month, from_month, format, remaining):
     """Prints the usage information for the given month or for the given
     range of months if --from-month is specified. The month must be specified
     in YYYY-MM format (e.g. "2014-05" for May 2014).
@@ -243,7 +271,8 @@ def usage(browser, month, from_month, format):
         def generate_formatted_strings():
             m = from_month
             while m <= month:
-                yield format_usage_info(browser.get_usage_info(m), format)
+                yield format_usage_info(browser.get_usage_info(m), format,
+                                        remaining)
                 m = m.next_month()            
         if format == 'json':
             print('[')
@@ -258,11 +287,12 @@ def usage(browser, month, from_month, format):
         if format == 'json':
             print('\n]')
     else:
-        print(format_usage_info(browser.get_usage_info(month), format))
+        print(format_usage_info(browser.get_usage_info(month), format,
+                                remaining))
 
 
 def main():
-    cli(auto_envvar_prefix='OLLEH')
+    cli()
 
 
 if __name__ == '__main__':
